#' Scord cards load metrics from verification suite
#'@description Scorecards function to load saved data files 
#'@param recipe .yml with the minimum information:
#'  system.filename a list of character strings defining the names of the 
#'system names following the arhcive.yml format from verification suite (to be added)
#'  reference.filename a list of character strings defining the names of the 
#'references following the archive.yml format from verification suite (to be added)
#'Pending to be test with more than one.
#'   var.filename a character string following the format from variable-dictionary.yml 
#'from verification suite (todo: multiple variables)
#'  start.year numeric indicating the start year of the reference period
#'  end.year numeric indicating the end year of the reference period
#'@param m a vector of character strings indicating the start months
#'@param input.path a character string indicating the path where metrics output files from 
#'verification suite where store (or any other compatible files)
## Todo:
## other parameters? leadtime in case only one session or few months want to be displayed 
#'@example
#'recipe_file <- "recipes/recipe_1_nadia_scorecards.yml"
#'recipe <- read_yaml(recipe_file)
#'loaded_metrics <- SC_load_metrics(system.filename = list('ecmwfs5','dwds21')),
#'                                  reference.filename = 'era5',
#'                                  var.filename = 'tas',
#'                                  start.year = 1993,
#'                                  end.year = 2016, 
#'                                  metric = 'mean_bias',
#'                                  m = sprintf("%02d", 1:12),
#'        input.path = '/esarchive/scratch/nmilders/scorecards_data/input_data')

SC_load_metrics <- function(sytem.filename, reference.filename, 
                            var.filename, start.year, end.year,
                            metrics = NULL,
                            m = sprintf("%02d", 1:12), 
                            input.path) {
  
  ## recipe: read recipe to obtain data of system, variable, start date etc.
  ## m: start_months to load from metrics outputs files
  ## metrics
  if (is.null(metric)) {
    metric <- c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr') ## TO DO: metric needs to be read from recipe
  }
  ## system.filenam
  if (!is.list(system.filename)) {
    if  (!is.character(system.filename)) {
      stop("Parameter 'system.filename' must be a character or a list.")
    }
  } 
  ## input.path: 
  if (is.null(input.path)) {
     stop("Parameter 'input.path' cannot be NULL.")
  }
  if (!is.character(input.path)) {
    stop("Parameter 'input.path must be a character string.")
  }
  if (length(input.path) > 1) {
    input.path <- input.path[1]
    warning("Parameter 'input.path' has length greater than 1 and only the ",
            "first element will be used.")
  }
  if (is.null(start.year)) {
    stop("Parameter 'start.year' cannot be NULL.")
  }
  if (is.null(end.year)) {
    stop("Parameter 'end.year' cannot be NULL.")
  }
  #system.filename <- list('ecmwfs5','dwds21') ## archive$System[[recipe$Analysis$Datasets$System$name]]$system_filename ## 'ecmwfs5' 
  #reference.filename <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_filename ## 'era5'
  #var.filename <- recipe$Analysis$Variables$name ## 'tas'
  ## start.year
  ## end.year
  period <- paste0(start.year, "-", end.year)
  
  #n.systems <- length(system.filename)
  n.start.months <- length(m) ## read from function input
  
  all_metrics <- list()
  for (sys in 1:length(system.filename)) {
    by_reference <- list()
    for (ref in 1:length(reference.filename)) {
  
  met <- initialfunction_read_data(start.months = m, 
                                   input.path = input.path, # recipe$Run$output, 
                                   system.filename = system.filename,
                                   reference.filename = reference.filename,
                                   var.filename = var.filename,
                                   period = period, 
                                   #mod.pressure = NULL, 
                                   #unitconversionfactor = initialfunction_conversionfactor(var.name, system.filename), 
                                   metrics = metrics,
                                   n.systems = length(system.filename))
    }
  }
}


##### Function to load data files #####

# metric is a vector indicating the metrics to load using the names in the file 
# metric is null by default and is currently hardcoded 

initialfunction_read_data <- function(start.months, input.path, system.filename, reference.filename, var.filename, period, m, metrics, n.systems){
  
  metrics_loaded <- list()
 

  ## Load data for each system
  #for(sys in 1:n.systems){
    lon <- NULL
    lat <- NULL
    met_by_sdate <- list()
    
    ## Load data for each start date
    for(m in start.months){
      
     ## netCDF data file path:
     print(paste0("Loading netCDF file: ", paste0(input.path, "/", system.filename[sys], "/", var.filename,
                                                "/scorecards_", system.filename[sys], "_", reference.filename, "_",
                                                 var.filename, "_", period, "_s", m, ".nc")))

      nc <- paste0(input.path, "/", system.filename[sys], "/", var.filename, 
                           "/scorecards_", system.filename[sys], "_", reference.filename, "_", 
                           var.filename, "_", period, "_s", m, #mod.pressure,
                           ".nc")
      
      ## Load data of each metric:
      mm <- as.numeric(m)
      met_by_sys <- NULL #list()
      
      for (kk in 1:length(metric)) {
         met <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = metric[kk], unlist = T, 
                                    drop_var_dim = T)},
                          error = function(cond) {
                            print("Missing file for start date")
                            message(cond)
                            return(NA)
                          })
         
          ## Load latitude and longitude data as attribute of system
          if (is.null(lon)) {
            lon <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = 'longitude', unlist = T, drop_var_dim = T)})
            lat <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = 'latitude', unlist = T, drop_var_dim = T)})
          }
         
         ## Save data for each metric
         attributes(met)$variables <- NULL
         met_by_sys <- abind::abind(met_by_sys, met, along = 4) #append(met_by_sys, list(met))
         names(met_by_sys)[[length(met_by_sys)]] <- metric[kk]
      } ## close loop on metric
      
    ## Save metric data for each start date  
    met_by_sdate <- append(met_by_sdate, list(met_by_sys))
    names(met_by_sdate)[[length(met_by_sdate)]] <- m
    } ## close loop on start months
    
  ## Save metric data by start date for each system  
  attributes(met_by_sdate)$lon <- lon
  attributes(met_by_sdate)$lat <- lat
  metrics_loaded <- append(metrics_loaded, list(met_by_sdate))
  names(metrics_loaded)[length(metrics_loaded)] <- system.filename[[sys]]
#  } ## close loop on sys
              
  return(metrics_loaded)
  
 } ## close function 

   
    


